from enum import Enum
from typing import Optional
from uuid import UUID

from pydantic import BaseModel


class Type(str, Enum):
    VOUCHER = "voucher"


class File(BaseModel):
    id: UUID
    voucherId: Optional[UUID] = None



from datetime import datetime
from enum import Enum
from pydantic import BaseModel
from uuid import UUID

class VoucherType(str, Enum):
    SALES_INVOICE = 'salesinvoice'
    SALES_CREDIT_NOTE = 'salesCreditNote'
    PURCHASE_INVOICE = 'purchaseInvoice'
    PURCAHSE_CREDIT_NOTE = 'purchaseCreditNote'

class VoucherStatus(str, Enum):
    UNCHECKED = 'unchecked'
    OPEN = 'open'

class TaxType(str, Enum):
    NET = 'net'
    GROSS = 'gross'

class VoucherItem(BaseModel):
    amount: float
    taxAmount: float
    taxRatePercent: float
    categoryId: UUID

class VoucherReadOnly(BaseModel):
    id: UUID
    organizationId: UUID
    createdDate: datetime
    updatedDate: datetime

class VoucherWritable(BaseModel):
    type: VoucherType
    voucherStatus: VoucherStatus | None
    voucherNumber: str | None
    voucherDate: datetime | None
    shippingDate: datetime | None
    dueDate: datetime | None
    totalGrossAmount: float
    totalTaxAmount: float
    taxType: TaxType
    useCollectiveContact: bool | None
    contactId: UUID | None
    remark: str | None
    voucherItems: list[VoucherItem]
    version: int | None
    files: list[str] | None
    def check_model(self) -> VoucherWritable: ...

class Voucher(VoucherReadOnly, VoucherWritable): ...



import html
from pathlib import Path
from typing import Optional, List, Dict, Union
from uuid import UUID
import httpx
from lexoffice_client.common import CreateResponse
from lexoffice_client.contact import Contact, ContactWritable
from lexoffice_client.posting_category import PostingCategoryReadOnly
from lexoffice_client.voucher import Voucher, VoucherWritable
from lexoffice_client.file import File, Type

from urllib.parse import urlencode


class LexofficeClient:
    def __init__(
        self, access_token: str, base_url: str = "https://api.lexoffice.io/v1"
    ):
        self.client = httpx.Client(
            headers={
                "Authorization": f"Bearer {access_token}",
                "Accept": "application/json",
            },
            base_url=base_url,
            timeout=10,
        )

    def ping(self):
        """Ping Lexoffice API and test the connection."""
        response = self.client.get("/ping")
        if not response.is_success:
            raise httpx.HTTPStatusError(
                f"Error pinging Lexoffice API: {response.status_code} - {response.text}",
                request=response.request,
                response=response,
            )

    def create_contact(self, contact: ContactWritable) -> CreateResponse:
        """Create a new contact.

        :param contact: The contact to create.
        :return: The response from the Lexoffice API.
        """

        response = self.client.post(
            "/contacts", json=contact.model_dump(exclude_none=True)
        )

        if not response.is_success:
            raise httpx.HTTPStatusError(
                f"Error creating contact: {response.status_code} - {response.text}",
                request=response.request,
                response=response,
            )

        return CreateResponse.model_validate(response.json())

    def retrieve_contact(self, contact_id: UUID) -> Contact:
        """Retrieve a contact by its ID.

        :param contact_id: The ID of the contact to retrieve.
        :return: The retrieved contact as a Contact object.
        """
        response = self.client.get(f"/contacts/{contact_id}")

        if not response.is_success:
            raise httpx.HTTPStatusError(
                f"Error retrieving contact: {response.status_code} - {response.text}",
                request=response.request,
                response=response,
            )

        return Contact.model_validate(response.json())

    def filter_contacts(
        self,
        email: Optional[str] = None,
        name: Optional[str] = None,
        number: Optional[int] = None,
        customer: Optional[bool] = None,
        vendor: Optional[bool] = None,
    ) -> List[Contact]:
        """Filter contacts by optional email, name, number, customer, and vendor.

        :param email: The email address to filter by (optional).
        :param name: The name to filter by (optional).
        :param number: The number of contacts to retrieve (optional).
        :param customer: Whether to filter by customer (optional).
        :param vendor: Whether to filter by vendor (optional).
        :return: A list of filtered contacts.
        """
        query_params: Dict[str, Union[str, None]] = {
            "email": html.escape(email) if email else None,
            "name": html.escape(name) if name else None,
            "number": str(number) if number is not None else None,
            "customer": str(customer).lower() if customer is not None else None,
            "vendor": str(vendor).lower() if vendor is not None else None,
        }
        query_params = {k: v for k, v in query_params.items() if v is not None}

        encoded_query = urlencode(query_params)
        response = self.client.get(f"/contacts?{encoded_query}")

        if not response.is_success:
            raise httpx.HTTPStatusError(
                f"Error filtering contacts: {response.status_code} - {response.text}",
                request=response.request,
                response=response,
            )

        content = response.json().get("content")
        if content is None:
            raise ValueError("Invalid response format: 'content' key not found.")
        return [Contact.model_validate(contact) for contact in content]

    def retrieve_posting_categories_list(self) -> List[PostingCategoryReadOnly]:
        """Retrieve a list of posting categories.

        :return: A list of PostingCategoryReadOnly objects.
        """
        response = self.client.get("/posting-categories")

        if not response.is_success:
            raise httpx.HTTPStatusError(
                f"Error retrieving posting categories: {response.status_code} - {response.text}",
                request=response.request,
                response=response,
            )

        content = response.json()
        if content is None:
            raise ValueError("Invalid response format: 'content' key not found.")
        return [
            PostingCategoryReadOnly.model_validate(category) for category in content
        ]

    def create_voucher(self, voucher: VoucherWritable) -> CreateResponse:
        """Create a new voucher.

        :param voucher: The voucher to create.
        :return: The response from the Lexoffice API.
        """
        response = self.client.post(
            "/vouchers", json=voucher.model_dump(exclude_none=True, mode="json")
        )

        if not response.is_success:
            raise httpx.HTTPStatusError(
                f"Error creating voucher: {response.status_code} - {response.text}",
                request=response.request,
                response=response,
            )

        return CreateResponse.model_validate(response.json())

    def retrieve_voucher(self, voucher_id: UUID) -> Voucher:
        """Retrieve a voucher by its ID.

        :param voucher_id: The ID of the voucher to retrieve.
        :return: The retrieved voucher as an Invoice object.
        """
        response = self.client.get(f"/vouchers/{voucher_id}")

        if not response.is_success:
            raise httpx.HTTPStatusError(
                f"Error retrieving voucher: {response.status_code} - {response.text}",
                request=response.request,
                response=response,
            )

        return Voucher.model_validate(response.json())

    def upload_file_to_voucher(
        self,
        voucher_id: UUID,
        file_path: Union[str, Path],
        upload_type: Type = Type.VOUCHER,
    ) -> File:
        file_path = Path(file_path)
        if not file_path.exists():
            raise FileNotFoundError(f"File not found: {file_path}")
        if file_path.stat().st_size > 5 * 1024 * 1024:
            raise ValueError("File size exceeds the 5MB limit.")

        files = {"file": open(file_path, "rb")}
        data = {"type": upload_type.value}

        response = self.client.post(
            f"/vouchers/{voucher_id}/files",
            files=files,
            data=data,
        )

        if not response.is_success:
            raise httpx.HTTPStatusError(
                f"Error uploading file: {response.status_code} - {response.text}",
                request=response.request,
                response=response,
            )

        return File.model_validate(response.json())



from enum import Enum
from pydantic import BaseModel, EmailStr as EmailStr
from uuid import UUID

class CountryCode(str, Enum):
    DE = "DE"

class Address(BaseModel):
    suppliment: str | None
    street: str | None
    zip: str | None
    city: str | None
    countryCode: CountryCode

class Addresses(BaseModel):
    billing: Address | None
    shipping: Address | None

class CompanyContactPerson(BaseModel):
    salutation: str | None
    firstName: str | None
    lastName: str
    emailAddress: EmailStr | None
    phoneNumber: str | None

class Company(BaseModel):
    allowTaxFreeInvoices: bool | None
    name: str
    taxNumber: str | None
    vatRegistrationId: str | None
    contactPersons: list[CompanyContactPerson] | None

class Person(BaseModel):
    salutation: str | None
    firstName: str | None
    lastName: str

class Role(BaseModel):
    number: int | None

class Roles(BaseModel):
    customer: Role | None
    vendor: Role | None
    def check_model(self) -> Roles: ...

class ContactReadOnly(BaseModel):
    id: UUID
    organizationId: UUID
    archived: bool

class ContactWritable(BaseModel):
    roles: Roles
    company: Company | None
    person: Person | None
    addresses: str | None
    def check_model(self) -> ContactWritable: ...

class Contact(ContactReadOnly, ContactWritable): ...



from enum import Enum
from uuid import UUID
from pydantic import BaseModel


class Type(str, Enum):
    INCOME = "income"
    OUTGO = "outgo"


class PostingCategoryReadOnly(BaseModel):
    id: UUID
    name: str
    type: Type
    contactRequired: bool
    splitAllowed: bool
    groupName: str



from _typeshed import Incomplete
from lexoffice_client.common import CreateResponse as CreateResponse
from lexoffice_client.contact import Contact as Contact, ContactWritable as ContactWritable
from lexoffice_client.voucher import Voucher as Voucher, VoucherWritable as VoucherWritable
from uuid import UUID

class LexofficeClient:
    client: Incomplete
    def __init__(self, access_token: str, base_url: str = 'https://api.lexoffice.io/v1') -> None: ...
    def ping(self) -> None: ...
    def create_contact(self, contact: ContactWritable) -> CreateResponse: ...
    def retrieve_contact(self, contact_id: UUID) -> Contact: ...
    def filter_contacts(self, email: str | None = None, name: str | None = None, number: int | None = None, customer: bool | None = None, vendor: bool | None = None) -> list[Contact]: ...
    def create_voucher(self, voucher: VoucherWritable) -> CreateResponse: ...
    def retrieve_voucher(self, voucher_id: UUID) -> Voucher: ...



from enum import Enum
from typing import List, Optional
from uuid import UUID
from pydantic import BaseModel, EmailStr, model_validator


class CountryCode(str, Enum):
    DE = "DE"


class Address(BaseModel):
    suppliment: Optional[str] = None
    street: Optional[str] = None
    zip: Optional[str] = None
    city: Optional[str] = None
    countryCode: CountryCode


class Addresses(BaseModel):
    billing: Optional[List[Address]] = None
    shipping: Optional[List[Address]] = None


class CompanyContactPerson(BaseModel):
    salutation: Optional[str] = None
    firstName: Optional[str] = None
    lastName: str
    emailAddress: Optional[EmailStr] = None
    phoneNumber: Optional[str] = None


class Company(BaseModel):
    allowTaxFreeInvoices: Optional[bool] = None
    name: str
    taxNumber: Optional[str] = None
    vatRegistrationId: Optional[str] = None
    contactPersons: Optional[List[CompanyContactPerson]] = None


class Person(BaseModel):
    salutation: Optional[str] = None
    firstName: Optional[str] = None
    lastName: str


class Role(BaseModel):
    number: Optional[int] = None


class Roles(BaseModel):
    customer: Optional[Role] = None
    vendor: Optional[Role] = None

    @model_validator(mode="after")
    def check_model(self) -> "Roles":
        if bool(self.customer) == bool(self.vendor):
            raise ValueError("Exactly one of 'customer' or 'vendor' must be set.")
        return self


class ContactReadOnly(BaseModel):
    id: UUID
    organizationId: UUID
    archived: bool


class ContactWritable(BaseModel):
    roles: Roles
    company: Optional[Company] = None
    person: Optional[Person] = None
    addresses: Optional[Addresses] = None

    @model_validator(mode="after")
    def check_model(self) -> "ContactWritable":
        if bool(self.company) == bool(self.person):
            raise ValueError("Exactly one of 'company' or 'person' must be set.")
        return self


class Contact(ContactReadOnly, ContactWritable):
    pass



from datetime import datetime
import unittest
from unittest.mock import patch, MagicMock
from uuid import uuid4
from lexoffice_client.client import CreateResponse, LexofficeClient
from lexoffice_client.contact import Contact, ContactWritable, Person, Role, Roles
from lexoffice_client.voucher import (
    TaxType,
    Voucher,
    VoucherItem,
    VoucherStatus,
    VoucherType,
    VoucherWritable,
)

ORGANIZATION_ID = uuid4()
CONTACT_ID = uuid4()
VOUCHER_ID = uuid4()
CATEGORY_ID = uuid4()

NOW = datetime.now()


class TestLexofficeClient(unittest.TestCase):
    def setUp(self):
        self.access_token = "test_token"
        self.client = LexofficeClient(access_token=self.access_token)

    @patch("httpx.Client.get")
    def test_ping(self, mock_get: MagicMock):
        mock_get.return_value = MagicMock(status_code=200)
        self.client.ping()
        mock_get.assert_called_once_with("/ping")

    @patch("httpx.Client.post")
    def test_create_contact(self, mock_post: MagicMock):
        contact = ContactWritable(
            roles=Roles(customer=Role()), person=Person(lastName="Test")
        )
        mock_response = MagicMock(status_code=201)
        mock_response.json.return_value = CreateResponse(
            id=CONTACT_ID,
            resourceUri=f"/contacts/{CONTACT_ID}",
            createdDate=NOW,
            updatedDate=NOW,
            version=1,
        )
        mock_post.return_value = mock_response

        result = self.client.create_contact(contact)
        mock_post.assert_called_once()
        self.assertEqual(result.id, CONTACT_ID)

    @patch("httpx.Client.get")
    def test_retrieve_contact(self, mock_get: MagicMock):
        mock_response = MagicMock(status_code=200)

        contact = Contact(
            id=CONTACT_ID,
            organizationId=ORGANIZATION_ID,
            roles=Roles(customer=Role()),
            person=Person(lastName="Test"),
            archived=False,
        )

        mock_response.json.return_value = contact.model_dump()
        mock_get.return_value = mock_response

        result = self.client.retrieve_contact(CONTACT_ID)
        mock_get.assert_called_once_with(f"/contacts/{CONTACT_ID}")
        self.assertEqual(result.id, CONTACT_ID)

    @patch("httpx.Client.get")
    def test_filter_contacts(self, mock_get: MagicMock):
        mock_response = MagicMock(status_code=200)
        mock_response.json.return_value = {"content": []}
        mock_get.return_value = mock_response

        result = self.client.filter_contacts(email="test@example.com")
        mock_get.assert_called_once()
        self.assertEqual(result, [])

    @patch("httpx.Client.post")
    def test_create_voucher(self, mock_post: MagicMock):
        voucher = VoucherWritable(
            type=VoucherType.SALES_INVOICE,
            voucherStatus=VoucherStatus.UNCHECKED,
            totalGrossAmount=119.0,
            totalTaxAmount=19.0,
            taxType=TaxType.GROSS,
            useCollectiveContact=True,
            voucherItems=[
                VoucherItem(
                    amount=100.0,
                    taxAmount=19.0,
                    taxRatePercent=19.0,
                    categoryId=CATEGORY_ID,
                )
            ],
        )
        mock_response = MagicMock(status_code=201)
        mock_response.json.return_value = CreateResponse(
            id=VOUCHER_ID,
            resourceUri=f"/vouchers/{VOUCHER_ID}",
            createdDate=NOW,
            updatedDate=NOW,
            version=1,
        )
        mock_post.return_value = mock_response

        result = self.client.create_voucher(voucher)
        mock_post.assert_called_once()
        self.assertEqual(result.id, VOUCHER_ID)

    @patch("httpx.Client.get")
    def test_retrieve_voucher(self, mock_get: MagicMock):
        mock_response = MagicMock(status_code=200)

        voucher = Voucher(
            id=VOUCHER_ID,
            organizationId=VOUCHER_ID,
            createdDate=NOW,
            updatedDate=NOW,
            type=VoucherType.SALES_INVOICE,
            voucherStatus=VoucherStatus.UNCHECKED,
            totalGrossAmount=119.0,
            totalTaxAmount=19.0,
            taxType=TaxType.GROSS,
            useCollectiveContact=True,
            voucherItems=[
                VoucherItem(
                    amount=100.0,
                    taxAmount=19.0,
                    taxRatePercent=19.0,
                    categoryId=uuid4(),
                )
            ],
        )

        mock_response.json.return_value = voucher.model_dump()
        mock_get.return_value = mock_response

        result = self.client.retrieve_voucher(VOUCHER_ID)
        mock_get.assert_called_once_with(f"/vouchers/{VOUCHER_ID}")
        self.assertEqual(result.id, VOUCHER_ID)


if __name__ == "__main__":
    unittest.main()



from datetime import datetime
from uuid import UUID
from pydantic import BaseModel


class CreateResponse(BaseModel):
    id: UUID
    resourceUri: str
    createdDate: datetime
    updatedDate: datetime
    version: int



from setuptools import setup, find_packages

setup(
    name="lexoffice-client",
    version="0.1.4",
    author="James Maguire",
    author_email="jvm986@gmail.com",
    description="A Python client for the Lexoffice API",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/jvm986/lexoffice-client",
    packages=find_packages(),
    include_package_data=True,
    package_data={
        "lexoffice_client": ["py.typed", "**/*.pyi"],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    install_requires=["email_validator", "httpx", "pydantic"],
)



from datetime import datetime
from enum import Enum
from typing import List, Optional
from uuid import UUID
from pydantic import BaseModel, model_validator


class VoucherType(str, Enum):
    SALES_INVOICE = "salesinvoice"
    SALES_CREDIT_NOTE = "salescreditnote"
    PURCHASE_INVOICE = "purchaseinvoice"
    PURCHASE_CREDIT_NOTE = "purchasecreditnote"


class VoucherStatus(str, Enum):
    UNCHECKED = "unchecked"
    OPEN = "open"


class TaxType(str, Enum):
    NET = "net"
    GROSS = "gross"


class VoucherItem(BaseModel):
    amount: float
    taxAmount: float
    taxRatePercent: float
    categoryId: UUID


class VoucherReadOnly(BaseModel):
    id: UUID
    organizationId: UUID
    createdDate: datetime
    updatedDate: datetime


class VoucherWritable(BaseModel):
    type: VoucherType
    voucherStatus: Optional[VoucherStatus] = None
    voucherNumber: Optional[str] = None
    voucherDate: Optional[datetime] = None
    shippingDate: Optional[datetime] = None
    dueDate: Optional[datetime] = None
    totalGrossAmount: float
    totalTaxAmount: float
    taxType: TaxType
    useCollectiveContact: Optional[bool] = None
    contactId: Optional[UUID] = None
    remark: Optional[str] = None
    voucherItems: List[VoucherItem]
    version: Optional[int] = None
    files: Optional[List[str]] = None

    @model_validator(mode="after")
    def check_model(self) -> "VoucherWritable":
        if self.voucherStatus != VoucherStatus.UNCHECKED:
            if (
                not self.voucherNumber
                or not self.voucherDate
                or not self.totalGrossAmount
                or not self.totalTaxAmount
            ):
                raise ValueError(
                    "voucherNumber, voucherDate, totalGrossAmount and totalTaxAmount are required when voucherStatus is not unchecked."
                )
        if not self.useCollectiveContact:
            if not self.contactId:
                raise ValueError(
                    "contactId is required when useCollectiveContact is false."
                )
        return self


class Voucher(VoucherReadOnly, VoucherWritable):
    pass
